// Copyright 2023 schukai GmbH
// SPDX-License-Identifier: AGPL-3.0

package document

import (
	"bytes"
	"errors"
	"gopkg.in/yaml.v3"
	"strings"
	"text/template"
	"time"
)

var dateFormat = "2006-01-02"

type textMetaStruct struct {
	text string
	meta document
}

type LocaleTime struct {
	time.Time
}

func (l *LocaleTime) UnmarshalYAML(unmarshal func(interface{}) error) error {
	var s string
	if err := unmarshal(&s); err != nil {
		return err
	}

	t, err := time.Parse(dateFormat, s)
	if err != nil {
		t, err = time.Parse("2006-01-02", s)
		if err != nil {
			return err
		}
		return err
	}

	l.Time = t

	return nil
}

func evaluateDocumentContent(tmpl GetComponentsTemplatesInterface, content []byte) (error, textMetaStruct) {

	origin := string(content)

	meta := ""
	text := ""

	if strings.Index(origin, "---") == 0 {
		origin = "\n" + origin
	}

	before, remaining, found := strings.Cut(origin, "\n---\n")
	if !found {

		req := textMetaStruct{}
		req.text = origin

		return nil, req
	}

	text += before

	a, b, found := strings.Cut(remaining, "\n...\n")
	if !found {
		a, b, found = strings.Cut(remaining, "\n---\n")

		if !found {

			a, b, found = strings.Cut(remaining, "\n...\n")

			if !found {

				a, b, found = strings.Cut(remaining, "\n---\n")

				if !found {
					return errors.New("the file does not contain a definition block"), textMetaStruct{}
				}

			}
		}

	}

	meta = a
	text += b

	t, err := template.New("overview").Funcs(getFuncMap(tmpl)).Parse(text)
	if err != nil {
		return err, textMetaStruct{}
	}

	var node yaml.Node
	err = yaml.Unmarshal([]byte(meta), &node)
	if err != nil {
		return err, textMetaStruct{}
	}

	data := document{}
	err = node.Decode(&data)
	if err != nil {
		return err, textMetaStruct{}
	}

	req := textMetaStruct{}
	req.meta = data

	req.meta.OriginNode = &node
	req.meta.OriginText = text

	var buffer bytes.Buffer
	err = t.Execute(&buffer, data)
	if err != nil {
		return err, textMetaStruct{}
	}

	req.text = buffer.String()

	return nil, req

}
